@@ -3,12 +3,11 @@ package Inline::Java::Class ;
 
 use strict ;
 
-$Inline::Java::Class::VERSION = '0.01' ;
+$Inline::Java::Class::VERSION = '0.20' ;
 
 use Carp ;
 
 
-
 my $INT_RE = '^[+-]?\d+$' ;
 my $FLOAT_RE = '^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$' ;
 
@@ -30,18 +29,26 @@ my $RANGE = {
 	},
 	'java.lang.Long' => {
 		REGEXP => $INT_RE,
-		MAX => 9223372036854775807,
-		MIN => -9223372036854775808,
+		MAX => 2147483647,
+		MIN => -2147483648,
+		# MAX => 9223372036854775807,
+		# MIN => -9223372036854775808,
 	},
 	'java.lang.Float' => {
 		REGEXP => $FLOAT_RE,
 		MAX => 3.4028235e38,
-		MIN => 1.4e-45,
+		MIN => -3.4028235e38,
+		POS_MIN	=> 1.4e-45,
+		NEG_MAX => -1.4e-45,
 	},
 	'java.lang.Double' => {
 		REGEXP => $FLOAT_RE,
-		MAX => 1.7976931348623157e308,
-		MIN => 4.9e-324,
+		MAX => 3.4028235e38,
+		MIN => -3.4028235e38,
+		# MAX => 1.7976931348623157e308,
+		# MIN => -1.7976931348623157e308,
+		POS_MIN => 4.9e-324,
+		NEG_MAX => -4.9e-324,
 	},
 } ;
 $RANGE->{byte} = $RANGE->{'java.lang.Byte'} ;
@@ -81,6 +88,7 @@ sub ValidateClassSplit {
 sub CastArguments {
 	my $args = shift ;
 	my $proto = shift ;
+	my $module = shift ;
 
 	Inline::Java::debug_obj($args) ;
 	Inline::Java::debug_obj($proto) ;
@@ -90,73 +98,184 @@ sub CastArguments {
 	}
 
 	my $ret = [] ;
+	my $score = 0 ;
 	for (my $i = 0 ; $i < scalar(@{$args}) ; $i++){
-		$ret->[$i] = CastArgument($args->[$i], $proto->[$i]) ;
+		my $arg = $args->[$i] ;
+		my $pro = $proto->[$i] ;
+		my @r = CastArgument($arg, $pro, $module) ;
+		$ret->[$i] = $r[0] ;
+		
+		$score += $r[1] ;
 	}
 
-	return $ret ;
+	return ($ret, $score) ;
 }
 
 
 sub CastArgument {
 	my $arg = shift ;
 	my $proto = shift ;
+	my $module = shift ;
 
 	ValidateClass($proto) ;
 
-	if ((ClassIsReference($proto))&&(! UNIVERSAL::isa($arg, "Inline::Java::Object"))){
-		# Here we allow scalars to be passed in place of java.lang.Object
-		# They will wrapped on the Java side.
-		if ($proto ne "java.lang.Object"){
-			croak "Can't convert $arg to object $proto" ;
-		}
-	}
-	if ((ClassIsPrimitive($proto))&&(ref($arg))){
-		croak "Can't convert $arg to primitive $proto" ;
-	}
+	my $arg_ori = $arg ;
+	my $proto_ori = $proto ;
 
-	if (ClassIsNumeric($proto)){
-		if (! defined($arg)){
-			return 0 ;
+	my $sub = sub {
+		my $array_type = undef ;
+		if (UNIVERSAL::isa($arg, "Inline::Java::Class::Cast")){
+			my $v = $arg->get_value() ;
+			$proto = $arg->get_type() ;
+			$array_type = $arg->get_array_type() ;
+			$arg = $v ;
 		}
-		my $re = $RANGE->{$proto}->{REGEXP} ;
-		my $min = $RANGE->{$proto}->{MIN} ;
-		my $max = $RANGE->{$proto}->{MAX} ;
-		Inline::Java::debug("min = $min, max = $max, val = $arg") ;
-		if ($arg =~ /$re/){
-			if (($arg >= $min)&&($arg <= $max)){
-				return $arg ;
+
+		if ((ClassIsReference($proto))&&(! UNIVERSAL::isa($arg, "Inline::Java::Object"))){
+			# Here we allow scalars to be passed in place of java.lang.Object
+			# They will wrapped on the Java side.
+			if (defined($arg)){
+				if (UNIVERSAL::isa($arg, "ARRAY")){
+					if (! UNIVERSAL::isa($arg, "Inline::Java::Array")){
+						my $an = new Inline::Java::Array::Normalizer($array_type || $proto, $arg) ;
+						my $flat = $an->FlattenArray() ; 
+						my $inline = Inline::Java::get_INLINE($module) ;
+						my $obj = Inline::Java::Object->__new($array_type || $proto, $inline, -1, $flat->[0], $flat->[1]) ;
+
+						# We need to create the array on the Java side, and then grab 
+						# the returned object.
+						$arg = new Inline::Java::Array($obj) ;
+					}
+					else{
+						Inline::Java::debug("argument is already an Inline::Java array") ;
+					}
+				}
+				else{
+					if (ref($arg)){
+						# We got some other type of ref...
+						croak "Can't convert $arg to object $proto" ;
+					}
+					else{
+						# Here we got a scalar
+						# Here we allow scalars to be passed in place of java.lang.Object
+						# They will wrapped on the Java side.
+						if ($proto ne "java.lang.Object"){
+							croak "Can't convert $arg to object $proto" ;
+						}
+					}
+				}
 			}
-			croak "$arg out of range for type $proto" ;
 		}
-		croak "Can't convert $arg to $proto" ;
-	}
-	elsif (ClassIsChar($proto)){
-		if (! defined($arg)){
-			return "\0" ;
+		if ((ClassIsPrimitive($proto))&&(ref($arg))){
+			croak "Can't convert $arg to primitive $proto" ;
 		}
-		if (length($arg) == 1){
-			return $arg ;
+
+		if (ClassIsNumeric($proto)){
+			if (! defined($arg)){
+				# undef gets lowest score since it can be passed
+				# as anything
+				return (0, 1) ;
+			}
+			my $re = $RANGE->{$proto}->{REGEXP} ;
+			my $min = $RANGE->{$proto}->{MIN} ;
+			my $max = $RANGE->{$proto}->{MAX} ;
+			Inline::Java::debug("min = $min, max = $max, val = $arg") ;
+			if ($arg =~ /$re/){
+				if (($arg >= $min)&&($arg <= $max)){
+					# number is a pretty precise match, but it's still
+					# guessing amongst the numeric types
+					return ($arg, 5.5) ;
+				}
+				croak "$arg out of range for type $proto" ;
+			}
+			croak "Can't convert $arg to $proto" ;
 		}
-		croak "Can't convert $arg to $proto" ;
-	}
-	elsif (ClassIsBool($proto)){
-		if ($arg){
-			return 1 ;
+		elsif (ClassIsChar($proto)){
+			if (! defined($arg)){
+				# undef gets lowest score since it can be passed
+				# as anything
+				return ("\0", 1) ;
+			}
+			if (length($arg) == 1){
+				# char is a pretty precise match
+				return ($arg, 5) ;
+			}
+			croak "Can't convert $arg to $proto" ;
+		}
+		elsif (ClassIsBool($proto)){
+			if (! defined($arg)){
+				# undef gets lowest score since it can be passed
+				# as anything
+				return (0, 1) ;
+			}
+			elsif (! $arg){
+				# bool gets lowest score since anything is a bool
+				return (0, 1) ;
+			}
+			else{
+				# bool gets lowest score since anything is a bool
+				return (1, 1) ;
+			}
+		}
+		elsif (ClassIsString($proto)){
+			if (! defined($arg)){
+				# undef gets lowest score since it can be passed
+				# as anything
+				return (undef, 1) ;
+			}
+			# string get almost lowest score since anything can match it
+			# except objects
+			if ($proto eq "java.lang.StringBuffer"){
+				# in case we have both protos, we want to give String
+				# the advantage
+				return ($arg, 1.75) ;
+			}
+			return ($arg, 2) ;
 		}
 		else{
-			return 0 ;
+			if (! defined($arg)){
+				# undef gets lowest score since it can be passed
+				# as anything
+				return ($arg, 1) ;
+			}
+
+			# Here the prototype calls for an object of type $proto
+			# We must ask Java if our object extends $proto		
+			if (ref($arg)){
+				my ($msg, $score) = $arg->__isa($proto) ;
+				if ($msg){
+					croak $msg ;
+				}
+				Inline::Java::debug("$arg is a $proto") ;
+
+				# a matching object, pretty good match, except if proto
+				# is java.lang.Object
+				if ($proto eq "java.lang.Object"){	
+					return ($arg, 1) ;
+				}
+				
+				# Here we deduce point the more our argument is "far"
+				# from the prototype.
+				return ($arg, 7 - ($score * 0.01)) ;
+			}
+
+			# Here we are passing a scalar as an object, this is pretty
+			# vague as well
+			return ($arg, 1) ;
+		}
+	} ;
+
+	my @ret = $sub->() ;
+	
+	if (UNIVERSAL::isa($arg_ori, "Inline::Java::Class::Cast")){
+		# It seems we had casted the variable to a specific type
+		if ($arg_ori->matches($proto_ori)){
+			Inline::Java::debug("Type cast match!") ;
+			$ret[1] = 10 ;
 		}
 	}
-	elsif (ClassIsString($proto)){
-		if (! defined($arg)){
-			return "" ;
-		}
-		return $arg ;
-	}
-	else{
-		return $arg ;
-	}
+
+	return @ret ;
 }
 
 
@@ -275,8 +394,66 @@ sub ClassIsArray {
 }
 
 
-1 ;
 
+######################## Inline::Java::Class::Cast ########################
+package Inline::Java::Class::Cast ;
+
+
+use Carp ;
+
+sub new {
+	my $class = shift ;
+	my $type = shift ;
+	my $value = shift ;
+	my $array_type = shift ;
+
+	if (UNIVERSAL::isa($value, "Inline::Java::Class::Cast")){
+		# This allows chaining
+		$value = $value->get_value() ;
+	}
+	
+	my $this = {} ;
+	$this->{cast} = Inline::Java::Class::ValidateClass($type) ;
+	$this->{value} = $value ;
+	$this->{array_type} = $array_type ;
+
+	bless($this, $class) ;
+	return $this ;
+}
+
+
+sub get_value {
+	my $this = shift ;
+
+	return $this->{value} ;
+}
+
+
+sub get_type {
+	my $this = shift ;
+
+	return $this->{cast} ;
+}
+
+sub get_array_type {
+	my $this = shift ;
+
+	return $this->{array_type} ;
+}
+
+
+sub matches {
+	my $this = shift ;
+	my $proto = shift ;
+
+	return ($proto eq $this->{cast}) ;
+}
+
+
+package Inline::Java::Class ;
+
+
+1 ;
 
 
 __DATA__
@@ -295,6 +472,11 @@ class InlineJavaClass {
 		Makes sure a class exists
 	*/
 	Class ValidateClass(String name) throws InlineJavaException {
+		Class pc = FindType(name) ;
+		if (pc != null){
+			return pc ;
+		}
+
 		try {
 			Class c = Class.forName(name) ;
 			return c ;
@@ -347,12 +529,14 @@ class InlineJavaClass {
 				if (num){
 					ijs.debug("  args is undef -> forcing to " + p.getName() + " 0") ;
 					ret = ijp.CreateObject(p, new Object [] {"0"}, new Class [] {String.class}) ;
+					ijs.debug("    result is " + ret.toString()) ;
 				}
 				else{
-					ijs.debug("  args is undef -> forcing to " + p.getName() + " ''") ;
-					ret = ijp.CreateObject(p, new Object [] {""}, new Class [] {String.class}) ;
+					ret = null ;
+					ijs.debug("  args is undef -> forcing to " + p.getName() + " " + ret) ;
+					ijs.debug("    result is " + ret) ;
+					// ijp.CreateObject(p, new Object [] {""}, new Class [] {String.class}) ;
 				}
-				ijs.debug("    result is " + ret.toString()) ;
 			}
 			else if (type.equals("scalar")){
 				String arg = ijp.pack((String)tokens.get(1)) ;
@@ -440,19 +624,8 @@ class InlineJavaClass {
 				String objid = (String)tokens.get(2) ;
 
 				Class c = ValidateClass(c_name) ;
-				// We need to check if c extends p
-				Class parent = c ;
-				boolean got_it = false ;
-				while (parent != null){
-					ijs.debug("    parent is " + parent.getName()) ;
-					if (parent == p){
-						got_it = true ;
-						break ;
-					}
-					parent = parent.getSuperclass() ;
-				}
 
-				if (got_it){
+				if (DoesExtend(c, p) > -1){
 					ijs.debug("    " + c.getName() + " is a kind of " + p.getName()) ;
 					// get the object from the hash table
 					Integer oid = new Integer(objid) ;
@@ -472,6 +645,25 @@ class InlineJavaClass {
 	}
 
 
+	/* 
+		Returns the number of levels that separate a from b
+	*/
+	int DoesExtend(Class a, Class b){
+		// We need to check if a extends b
+		Class parent = a ;
+		int level = 0 ;
+		while (parent != null){
+			ijs.debug("    parent is " + parent.getName()) ;
+			if (parent == b){
+				return level ;
+			}
+			level++ ;
+			parent = parent.getSuperclass() ;
+		}
+
+		return -1 ;
+	}
+
 
 	/*
 		Finds the wrapper class for the passed primitive type.
@@ -508,6 +700,57 @@ class InlineJavaClass {
 	}
 
 
+	/*
+		Finds the primitive type class for the passed primitive type name.
+	*/
+	Class FindType (String name){
+		String [] list = {
+			"byte",
+			"short",
+			"int",
+			"long",
+			"float",
+			"double",
+			"boolean",
+			"char",
+			"B",
+			"S",
+			"I",
+			"J",
+			"F",
+			"D",
+			"Z",
+			"C",
+		} ;
+		Class [] listc = {
+			byte.class,
+			short.class,
+			int.class,
+			long.class,
+			float.class,
+			double.class,
+			boolean.class,
+			char.class,
+			byte.class,
+			short.class,
+			int.class,
+			long.class,
+			float.class,
+			double.class,
+			boolean.class,
+			char.class,
+		} ;
+
+		for (int i = 0 ; i < list.length ; i++){
+			if (name.equals(list[i])){
+				return listc[i] ;
+			}
+		}
+
+		return null ;
+	}
+
+
 	boolean ClassIsPrimitive (Class p){
 		String name = p.getName() ;
 
@@ -633,4 +876,17 @@ class InlineJavaClass {
 
 		return true ;
 	}
+
+	boolean ClassIsArray (Class p){
+		String name = p.getName() ;
+
+		if ((ClassIsReference(p))&&(name.startsWith("["))){
+			ijs.debug("  class " + name + " is array") ;
+			return true ;
+		}
+
+		return false ;
+	}
+
 }
+
