@@ -3,7 +3,7 @@ package Inline::Java::Protocol ;
 
 use strict ;
 
-$Inline::Java::Protocol::VERSION = '0.20' ;
+$Inline::Java::Protocol::VERSION = '0.21' ;
 
 use Inline::Java::Object ;
 use Inline::Java::Array ;
@@ -24,21 +24,6 @@ sub new {
 }
 
 
-sub SetClassPath {
-	my $this = shift ;
-	my $classpath = shift ;
-
-	Inline::Java::debug("setting classpath") ;
-
-	my $data = join(" ", 
-		"set_classpath", 
-		$this->ValidateArgs([$classpath]),
-	) ;
-
-	return $this->Send($data, 1) ;
-}
-
-
 sub Report {
 	my $this = shift ;
 	my $classes = shift ;
@@ -288,16 +273,18 @@ sub Send {
 			return undef ;
 		}
 		else{
-			my $obj = undef ;
 			my $inline = Inline::Java::get_INLINE($this->{module}) ;
+			my $perl_class = Inline::Java::java2perl($inline->{pkg}, $class) ;
+			if (! Inline::Java::known_to_perl($inline->{pkg}, $class)){
+				if ($inline->{Java}->{AUTOSTUDY}){
+					$inline->_study([$class]) ;
+				}
+				else{
+					$perl_class = "Inline::Java::Object" ;
+				}
+			}
 
-			my $perl_class = Inline::Java::known_to_perl($inline->{pkg}, $class) ;
-			if ($perl_class){
-				$obj = $perl_class->__new($class, $inline, $id) ;
-			}
-			else{
-				$obj = Inline::Java::Object->__new($class, $inline, $id) ;
-			}
+			my $obj = $perl_class->__new($class, $inline, $id) ;
 
 			Inline::Java::debug("checking if stub is array...") ;
 			if (Inline::Java::Class::ClassIsArray($class)){
@@ -371,9 +358,6 @@ class InlineJavaProtocol {
 		else if (c.equals("isa")){
 			ISA(st) ;
 		}
-		else if (c.equals("set_classpath")){
-			SetClassPath(st) ;
-		}
 		else if (c.equals("create_object")){
 			CreateJavaObject(st) ;
 		}
@@ -393,7 +377,7 @@ class InlineJavaProtocol {
 		Returns a report on the Java classes, listing all public methods
 		and members
 	*/
-	void Report(StringTokenizer st){
+	void Report(StringTokenizer st) throws InlineJavaException {
 		StringBuffer pw = new StringBuffer() ;
 
 		StringTokenizer st2 = new StringTokenizer(st.nextToken(), ":") ;
@@ -404,65 +388,58 @@ class InlineJavaProtocol {
 		ArrayList class_list = new ArrayList() ;
 		while (st3.hasMoreTokens()){
 			String c = st3.nextToken() ;
-			ijs.debug("reporting for " + c) ;
 			class_list.add(class_list.size(), c) ;
 		}
 
-		try {
-			for (int i = 0 ; i < class_list.size() ; i++){
-				String name = (String)class_list.get(i) ;
-				if (! name.startsWith("InlineJavaServer")){
-					Class c = Class.forName(name) ;
-															
-					pw.append("class " + c.getName() + "\n") ;
-					Constructor constructors[] = c.getConstructors() ;
-					Method methods[] = c.getMethods() ;
-					Field fields[] = c.getFields() ;
+		for (int i = 0 ; i < class_list.size() ; i++){
+			String name = (String)class_list.get(i) ;
+			Class c = ijc.ValidateClass(name) ;
 
-					for (int j = 0 ; j < constructors.length ; j++){
-						Constructor x = constructors[j] ;
-						String sign = CreateSignature(x.getParameterTypes()) ;
-						Class decl = x.getDeclaringClass() ;
-						pw.append("constructor" + " " + sign + "\n") ;
-					}
-					for (int j = 0 ; j < methods.length ; j++){
-						Method x = methods[j] ;
-						String stat = (Modifier.isStatic(x.getModifiers()) ? " static " : " instance ") ;
-						String sign = CreateSignature(x.getParameterTypes()) ;
-						Class decl = x.getDeclaringClass() ;
-						pw.append("method" + stat + decl.getName() + " " + x.getName() + sign + "\n") ;
-					}
-					for (int j = 0 ; j < fields.length ; j++){
-						Field x = fields[j] ;
-						String stat = (Modifier.isStatic(x.getModifiers()) ? " static " : " instance ") ;
-						Class decl = x.getDeclaringClass() ;
-						Class type = x.getType() ;
-						pw.append("field" + stat + decl.getName() + " " + x.getName() + " " + type.getName() + "\n") ;
-					}
+			ijs.debug("reporting for " + c) ;
+													
+			pw.append("class " + c.getName() + "\n") ;
+			Constructor constructors[] = c.getConstructors() ;
+			Method methods[] = c.getMethods() ;
+			Field fields[] = c.getFields() ;
+
+			int pub = c.getModifiers() & Modifier.PUBLIC ;
+			if (pub != 0){
+				// If the class is public and has no constructors,
+				// we provide a default no-arg constructors.
+				if (c.getDeclaredConstructors().length == 0){
+					String noarg_sign = CreateSignature(new Class [] {}) ;
+					pw.append("constructor " + noarg_sign + "\n") ;	
 				}
 			}
-		}
-		catch (ClassNotFoundException e){
-			System.err.println("Can't find class: " + e.getMessage()) ;
-			System.exit(1) ;
+			for (int j = 0 ; j < constructors.length ; j++){
+				Constructor x = constructors[j] ;
+				Class params[] = x.getParameterTypes() ;
+				String sign = CreateSignature(params) ;
+				Class decl = x.getDeclaringClass() ;
+				pw.append("constructor " + sign + "\n") ;
+			}
+
+			for (int j = 0 ; j < methods.length ; j++){
+				Method x = methods[j] ;
+				String stat = (Modifier.isStatic(x.getModifiers()) ? " static " : " instance ") ;
+				String sign = CreateSignature(x.getParameterTypes()) ;
+				Class decl = x.getDeclaringClass() ;
+				pw.append("method" + stat + decl.getName() + " " + x.getName() + sign + "\n") ;
+			}
+
+			for (int j = 0 ; j < fields.length ; j++){
+				Field x = fields[j] ;
+				String stat = (Modifier.isStatic(x.getModifiers()) ? " static " : " instance ") ;
+				Class decl = x.getDeclaringClass() ;
+				Class type = x.getType() ;
+				pw.append("field" + stat + decl.getName() + " " + x.getName() + " " + type.getName() + "\n") ;
+			}
 		}
 
 		SetResponse(pw.toString()) ;
 	}
 
 
-	void SetClassPath(StringTokenizer st) throws InlineJavaException {
-		String classpath = st.nextToken() ;
-		StringTokenizer st2 = new StringTokenizer(classpath, ":") ;
-		st2.nextToken() ;
-
-		String prop = pack(st2.nextToken()) ;
-		System.setProperty("java.class.path", prop) ;
-
-		SetResponse(null) ;
-	}
-
-
 	void ISA(StringTokenizer st) throws InlineJavaException {
 		int id = Integer.parseInt(st.nextToken()) ;
 
@@ -491,7 +468,6 @@ class InlineJavaProtocol {
 
 		if (! ijc.ClassIsArray(c)){
 			ArrayList f = ValidateMethod(true, c, class_name, st) ;
-			Constructor con = (Constructor)f.get(0) ;
 			Object p[] = (Object [])f.get(1) ;
 			Class clist[] = (Class [])f.get(2) ;
 
@@ -701,8 +677,14 @@ class InlineJavaProtocol {
 		String name = p.getName() ;
 		Object ret = null ;
 		try {
-			Constructor con = (Constructor)p.getConstructor(proto) ;
-			ret = con.newInstance(args) ;
+			// This will allow usage of the default no-arg constructor
+			if (proto.length == 0){
+				ret = p.newInstance() ;
+			}
+			else{
+				Constructor con = (Constructor)p.getConstructor(proto) ;
+				ret = con.newInstance(args) ;
+			}
 		}
 		catch (NoSuchMethodException e){
 			throw new InlineJavaException("Constructor for class " + name + " with signature " + ijs.CreateSignature(proto) + " not found: " + e.getMessage()) ;
@@ -774,10 +756,18 @@ class InlineJavaProtocol {
 		// Now we got a list of matching methods. 
 		// We have to figure out which one we will call.
 		if (ml.size() == 0){
-			throw new InlineJavaException(
-				(constructor ? "Constructor " : "Method ") + 
-				name + " for class " + c.getName() + " with signature " +
-				signature + " not found") ;
+			// Nothing matched. Maybe we got a default constructor
+			if ((constructor)&&(signature.equals("()"))){
+				ret.add(0, null) ;
+				ret.add(1, new Object [] {}) ;
+				ret.add(2, new Class [] {}) ;
+			}
+			else{
+				throw new InlineJavaException(
+					(constructor ? "Constructor " : "Method ") + 
+					name + " for class " + c.getName() + " with signature " +
+					signature + " not found") ;
+			}
 		}
 		else if (ml.size() == 1){
 			// Now we need to force the arguments received to match
